@@ -8,7 +8,7 @@
 
 #include "musiclistenersmanager.h"
 
-#include <KI18n/KLocalizedString>
+#include <KLocalizedString>
 
 #include <QtConcurrent>
 
@@ -239,7 +239,7 @@ QString TrackMetadataModel::fileUrl() const
 QUrl TrackMetadataModel::coverUrl() const
 {
     if (mCoverImage.isEmpty()) {
-        return QUrl(QStringLiteral("image://icon/media-optical-audio"));
+        return QUrl(QStringLiteral("image://icon/media-default-album"));
     } else {
         return mCoverImage;
     }
@@ -262,9 +262,7 @@ qulonglong TrackMetadataModel::databaseId() const
 
 void TrackMetadataModel::trackData(const TrackMetadataModel::TrackDataType &trackData)
 {
-    if ((mDatabaseId != 0 && trackData.databaseId() != mDatabaseId) ||
-            (!mFileUrl.isEmpty() && trackData.resourceURI() != mFileUrl) ||
-            (!mFullData.isEmpty() && trackData.databaseId() != mFullData.databaseId())) {
+    if (!mFileUrl.isEmpty() && trackData.resourceURI() != mFileUrl) {
         return;
     }
 
@@ -299,7 +297,9 @@ void TrackMetadataModel::fillDataFromTrackData(const TrackMetadataModel::TrackDa
     filterDataFromTrackData();
     endResetModel();
 
-    fetchLyrics();
+    if (trackData.hasDatabaseId()) {
+        fetchLyrics();
+    }
 
     mDatabaseId = trackData[DataTypes::DatabaseIdRole].toULongLong();
     Q_EMIT databaseIdChanged();
@@ -581,8 +581,10 @@ bool TrackMetadataModel::metadataExists(DataTypes::ColumnsRoles metadataRole) co
 
 void TrackMetadataModel::fetchLyrics()
 {
-    auto lyricicsValue = QtConcurrent::run(QThreadPool::globalInstance(), [=]() {
-        auto trackData = mFileScanner.scanOneFile(mFullData[DataTypes::ResourceRole].toUrl());
+    auto fileUrl = mFullData[DataTypes::ResourceRole].toUrl();
+    auto lyricicsValue = QtConcurrent::run(QThreadPool::globalInstance(), [fileUrl, this]() {
+        auto locker = QMutexLocker(&mFileScannerMutex);
+        auto trackData = mFileScanner.scanOneFile(fileUrl);
         if (!trackData.lyrics().isEmpty()) {
             return trackData.lyrics();
         }
