@@ -12,12 +12,18 @@
 #include "viewsLogging.h"
 #include "models/datamodel.h"
 #include "models/gridviewproxymodel.h"
+
+#if KF5KIO_FOUND
 #include "models/filebrowsermodel.h"
 #include "models/filebrowserproxymodel.h"
+#endif
 
-#include <KI18n/KLocalizedString>
+#include "elisa_settings.h"
+
+#include <KLocalizedString>
 
 #include <QQmlEngine>
+#include <QMetaEnum>
 
 class ViewManagerPrivate
 {
@@ -32,7 +38,10 @@ public:
                              ElisaUtils::FilterById,
                              ElisaUtils::Track,
                              Qt::DisplayRole,
+                             {Qt::DisplayRole},
+                             {i18n("Title")},
                              Qt::AscendingOrder,
+                             {i18n("A-Z"), i18n("Z-A")},
                              ViewManager::SingleAlbum,
                              ViewManager::DiscHeaders,
                              ViewManager::IsTrack,
@@ -43,6 +52,11 @@ public:
                              ViewManager::GenericDataModel,
                              ElisaUtils::FilterByGenre,
                              ElisaUtils::Artist,
+                             Qt::DisplayRole,
+                             {Qt::DisplayRole},
+                             {i18n("Name")},
+                             Qt::AscendingOrder,
+                             {i18n("A-Z"), i18n("Z-A")},
                              QUrl{QStringLiteral("image://icon/view-media-artist")},
                              ViewManager::DelegateWithoutSecondaryText,
                              ViewManager::ViewHideRating,
@@ -53,7 +67,12 @@ public:
                               ViewManager::GenericDataModel,
                               ElisaUtils::FilterByArtist,
                               ElisaUtils::Album,
-                              QUrl{QStringLiteral("image://icon/media-optical-audio")},
+                              DataTypes::TitleRole,
+                              {DataTypes::TitleRole, DataTypes::GenreRole, DataTypes::YearRole},
+                              {i18n("Title"), i18n("Genre"), i18n("Year")},
+                              Qt::AscendingOrder,
+                              {i18n("A-Z"), i18n("Z-A"), i18n("A-Z"), i18n("Z-A"), i18n("Oldest First"), i18n("Newest First")},
+                              QUrl{QStringLiteral("image://icon/media-default-album")},
                               ViewManager::DelegateWithSecondaryText,
                               ViewManager::ViewShowRating,
                               ViewManager::IsFlatModel}},
@@ -63,13 +82,20 @@ public:
                                  ViewManager::FileBrowserModel,
                                  ElisaUtils::FilterByPath,
                                  ElisaUtils::FileName,
+                                 Qt::DisplayRole,
+                                 {Qt::DisplayRole},
+                                 {i18n("Name")},
+                                 Qt::AscendingOrder,
+                                 {i18n("A-Z"), i18n("Z-A")},
                                  QUrl{QStringLiteral("image://icon/folder")},
                                  ViewManager::DelegateWithoutSecondaryText,
                                  ViewManager::ViewHideRating,
                                  ViewManager::IsFlatModel}},
     };
 
-    int mViewIndex = 0;
+    int mViewIndex = -1;
+
+    int mInitialIndex = -1;
 
     QList<ViewParameters> mViewParametersStack = (mViewsListData ? QList<ViewParameters>{mViewsListData->viewParameters(0)} : QList<ViewParameters>{});
 
@@ -87,6 +113,11 @@ int ViewManager::viewIndex() const
     return d->mViewIndex;
 }
 
+int ViewManager::initialIndex() const
+{
+    return d->mInitialIndex;
+}
+
 ViewsListData *ViewManager::viewsData() const
 {
     return d->mViewsListData;
@@ -98,11 +129,7 @@ void ViewManager::openView(int viewIndex)
 {
     qCDebug(orgKdeElisaViews()) << "ViewManager::openView" << viewIndex << d->mViewParametersStack.size() << d->mViewsListData;
 
-    if (!d->mViewsListData) {
-        return;
-    }
-
-    if (!d->mViewParametersStack.size()) {
+    if (!d->mViewsListData || d->mViewsListData->isEmpty() || !d->mViewsListData->isFullyInitialized()) {
         return;
     }
 
@@ -112,9 +139,9 @@ void ViewManager::openView(int viewIndex)
 
     const auto &viewParameters = d->mViewsListData->viewParameters(viewIndex);
 
-    qCDebug(orgKdeElisaViews()) << "ViewManager::openView" << "selected view";
+    qCDebug(orgKdeElisaViews()) << "ViewManager::openView" << "selected view" << viewIndex;
 
-    if (viewParameters != d->mViewParametersStack.back()) {
+    if (d->mViewParametersStack.isEmpty() || viewParameters != d->mViewParametersStack.back()) {
         qCDebug(orgKdeElisaViews()) << "ViewManager::openView" << "changing view";
         d->mViewIndex = viewIndex;
         Q_EMIT viewIndexChanged();
@@ -141,7 +168,7 @@ void ViewManager::openChildView(const DataTypes::MusicDataType &fullData)
         return;
     }
 
-    if (!d->mViewsListData) {
+    if (!d->mViewsListData || d->mViewsListData->isEmpty() || !d->mViewsListData->isFullyInitialized()) {
         return;
     }
 
@@ -227,6 +254,11 @@ void ViewManager::openArtistView(const QString &artist)
     }
 }
 
+void ViewManager::openNowPlaying()
+{
+    openView(0);
+}
+
 void ViewManager::viewIsLoaded()
 {
     qCDebug(orgKdeElisaViews()) << "ViewManager::viewIsLoaded" << d->mViewParametersStack.size()
@@ -258,9 +290,14 @@ void ViewManager::openViewFromData(const ViewParameters &viewParamaters)
     {
     case FileBrowserModel:
     {
+#if KF5KIO_FOUND
         newModel = new ::FileBrowserModel;
         auto *realProxyModel = new FileBrowserProxyModel;
         proxyModel = realProxyModel;
+#else
+        newModel = nullptr;
+        proxyModel = nullptr;
+#endif
         break;
     }
     case GenericDataModel:
@@ -294,6 +331,10 @@ void ViewManager::openViewFromData(const ViewParameters &viewParamaters)
                                                                          viewParamaters.mMainImage, viewParamaters.mDataType,
                                                                          newModel, proxyModel, viewParamaters.mFallbackItemIcon,
                                                                          viewParamaters.mDataFilter,
+                                                                         computePreferredSortRole(viewParamaters.mSortRole),
+                                                                         viewParamaters.mSortRoles, viewParamaters.mSortRoleNames,
+                                                                         computePreferredSortOrder(viewParamaters.mSortOrder),
+                                                                         viewParamaters.mSortOrderNames,
                                                                          viewParamaters.mViewCanBeRated, viewParamaters.mShowSecondaryTextOnDelegates,
                                                                          viewParamaters.mIsTreeModel);
 
@@ -317,7 +358,10 @@ void ViewManager::openViewFromData(const ViewParameters &viewParamaters)
                                                                          viewParamaters.mMainTitle, viewParamaters.mSecondaryTitle,
                                                                          viewParamaters.mMainImage, viewParamaters.mDataType,
                                                                          newModel, proxyModel, viewParamaters.mDataFilter,
-                                                                         viewParamaters.mSortRole, viewParamaters.mSortOrder,
+                                                                         computePreferredSortRole(viewParamaters.mSortRole),
+                                                                         viewParamaters.mSortRoles, viewParamaters.mSortRoleNames,
+                                                                         computePreferredSortOrder(viewParamaters.mSortOrder),
+                                                                         viewParamaters.mSortOrderNames,
                                                                          viewParamaters.mAlbumCardinality, viewParamaters.mAlbumViewStyle,
                                                                          viewParamaters.mRadioSpecificStyle, viewParamaters.mIsTreeModel);
 
@@ -363,6 +407,76 @@ void ViewManager::applyFilter(ViewParameters &nextViewParameters,
     }
 }
 
+QString ViewManager::buildViewId() const
+{
+    const auto &entryTypeMetaEnum = QMetaEnum::fromType<ElisaUtils::PlayListEntryType>();
+    QString viewId;
+
+    for (const auto &oneView : std::as_const(d->mViewParametersStack)) {
+        viewId += QString::fromLatin1(entryTypeMetaEnum.valueToKey(oneView.mDataType)) + QStringLiteral("::");
+    }
+
+    return viewId;
+}
+
+QStringList::iterator ViewManager::findViewPreference(QStringList &list, const QString &viewId) const
+{
+    auto itViewPreference = list.end();
+    for(itViewPreference = list.begin(); itViewPreference != list.end(); ++itViewPreference) {
+        auto parts = itViewPreference->splitRef(QStringLiteral("=="));
+        if (parts.size() != 2) {
+            continue;
+        }
+        if (parts[0] == viewId) {
+            break;
+        }
+    }
+
+    return itViewPreference;
+}
+
+Qt::SortOrder ViewManager::computePreferredSortOrder(Qt::SortOrder initialValue) const
+{
+    auto currentSortOrderPreferences = Elisa::ElisaConfiguration::sortOrderPreferences();
+    auto viewId = buildViewId();
+    auto itViewPreference = findViewPreference(currentSortOrderPreferences, viewId);
+
+    if (itViewPreference != currentSortOrderPreferences.end()) {
+        auto result = itViewPreference->splitRef(QStringLiteral("=="));
+        if (result.size() == 2) {
+            const auto &sortOrderMetaEnum = QMetaEnum::fromType<Qt::SortOrder>();
+            bool conversionOk;
+            auto newValue = static_cast<Qt::SortOrder>(sortOrderMetaEnum.keyToValue(result[1].toLatin1().data(), &conversionOk));
+            if (conversionOk) {
+                initialValue = newValue;
+            }
+        }
+    }
+
+    return initialValue;
+}
+
+int ViewManager::computePreferredSortRole(int initialValue) const
+{
+    auto currentSortRolePreferences = Elisa::ElisaConfiguration::sortRolePreferences();
+    auto viewId = buildViewId();
+    auto itViewPreference = findViewPreference(currentSortRolePreferences, viewId);
+
+    if (itViewPreference != currentSortRolePreferences.end()) {
+        auto result = itViewPreference->splitRef(QStringLiteral("=="));
+        if (result.size() == 2) {
+            const auto &sortRoleMetaEnum = QMetaEnum::fromType<DataTypes::ColumnsRoles>();
+            bool conversionOk;
+            auto newValue = static_cast<DataTypes::ColumnsRoles>(sortRoleMetaEnum.keyToValue(result[1].toLatin1().data(), &conversionOk));
+            if (conversionOk) {
+                initialValue = newValue;
+            }
+        }
+    }
+
+    return initialValue;
+}
+
 void ViewManager::goBack()
 {
     qCDebug(orgKdeElisaViews()) << "ViewManager::goBack" << d->mViewParametersStack.size()
@@ -391,12 +505,90 @@ void ViewManager::setViewsData(ViewsListData *viewsData)
     d->mViewsListData = viewsData;
     Q_EMIT viewsDataChanged();
 
-    if (d->mViewsListData) {
-        d->mViewParametersStack = {d->mViewsListData->viewParameters(d->mViewIndex)};
+    if (d->mViewsListData && (d->mViewIndex < 0 || d->mViewIndex >= d->mViewsListData->count())) {
+        d->mViewIndex = d->mInitialIndex;
+    }
 
+    if (d->mViewsListData && d->mViewIndex >= 0 && d->mViewIndex < d->mViewsListData->count() && d->mViewsListData->isFullyInitialized()) {
+        openView(d->mViewIndex);
+    }
+
+    if (d->mViewsListData) {
+        connect(d->mViewsListData, &ViewsListData::isFullyInitializedChanged,
+                this, &ViewManager::openInitialView);
+    }
+}
+
+void ViewManager::setInitialIndex(int newIndex)
+{
+    if (d->mInitialIndex == newIndex) {
+        return;
+    }
+
+    d->mInitialIndex = newIndex;
+    Q_EMIT initialIndexChanged();
+
+    if (d->mViewsListData && (d->mViewIndex < 0 || d->mViewIndex >= d->mViewsListData->count())) {
+        d->mViewIndex = d->mInitialIndex;
+    }
+
+    if (d->mViewsListData && d->mViewIndex >= 0 && d->mViewIndex < d->mViewsListData->count() && d->mViewsListData->isFullyInitialized()) {
         openView(d->mViewIndex);
     }
 }
 
+void ViewManager::openInitialView()
+{
+    openView(d->mInitialIndex);
+}
+
+void ViewManager::sortOrderChanged(Qt::SortOrder sortOrder)
+{
+    auto currentSortOrderPreferences = Elisa::ElisaConfiguration::sortOrderPreferences();
+
+    auto viewId = buildViewId();
+    auto itViewPreference = findViewPreference(currentSortOrderPreferences, viewId);
+
+    const auto &sortOrderMetaEnum = QMetaEnum::fromType<Qt::SortOrder>();
+    auto enumStringValue = sortOrderMetaEnum.valueToKey(sortOrder);
+    if (!enumStringValue) {
+        return;
+    }
+    QString newSortOrderPreference = viewId + QStringLiteral("==") + QString::fromLatin1(enumStringValue);
+
+    if (itViewPreference != currentSortOrderPreferences.end()) {
+        (*itViewPreference) = newSortOrderPreference;
+    } else {
+        currentSortOrderPreferences.push_back(newSortOrderPreference);
+    }
+
+    Elisa::ElisaConfiguration::setSortOrderPreferences(currentSortOrderPreferences);
+    Elisa::ElisaConfiguration::self()->save();
+}
+
+void ViewManager::sortRoleChanged(int sortRole)
+{
+    auto currentSortRolePreferences = Elisa::ElisaConfiguration::sortRolePreferences();
+
+    auto viewId = buildViewId();
+    auto itViewPreference = findViewPreference(currentSortRolePreferences, viewId);
+
+    const auto &sortRoleMetaEnum = QMetaEnum::fromType<DataTypes::ColumnsRoles>();
+    auto enumStringValue = sortRoleMetaEnum.valueToKey(static_cast<DataTypes::ColumnsRoles>(sortRole));
+    if (!enumStringValue) {
+        return;
+    }
+    QString newSortRolePreference = viewId + QStringLiteral("==") + QString::fromLatin1(enumStringValue);
+
+    if (itViewPreference != currentSortRolePreferences.end()) {
+        (*itViewPreference) = newSortRolePreference;
+    } else {
+        currentSortRolePreferences.push_back(newSortRolePreference);
+    }
+
+    Elisa::ElisaConfiguration::setSortRolePreferences(currentSortRolePreferences);
+    Elisa::ElisaConfiguration::self()->save();
+}
+
 
 #include "moc_viewmanager.cpp"
