@@ -12,7 +12,7 @@
 
 #include "viewsLogging.h"
 
-#include <KI18n/KLocalizedString>
+#include <KLocalizedString>
 
 #include <QUrl>
 #include <QDir>
@@ -64,7 +64,7 @@ public:
                                                {i18n("Title"), i18n("Artist"), i18n("Genre"), i18n("Year")},
                                                Qt::AscendingOrder,
                                                {i18n("A-Z"), i18n("Z-A"), i18n("A-Z"), i18n("Z-A"), i18n("A-Z"), i18n("Z-A"), i18n("Oldest First"), i18n("Newest First")},
-                                               QUrl{QStringLiteral("image://icon/media-optical-audio")},
+                                               QUrl{QStringLiteral("image://icon/media-default-album")},
                                                ViewManager::DelegateWithSecondaryText,
                                                ViewManager::ViewShowRating,
                                                ViewManager::IsFlatModel},
@@ -118,12 +118,12 @@ public:
                                                ViewManager::ViewHideRating,
                                                ViewManager::IsFlatModel},
                                               {{i18nc("Title of the file browser view", "Files")},
-                                               QUrl{QStringLiteral("image://icon/folder")},
+                                               QUrl{QStringLiteral("image://icon/document-open-folder")},
                                                ViewManager::GridView,
                                                ViewManager::FileBrowserModel,
                                                ElisaUtils::NoFilter,
                                                ElisaUtils::FileName,
-                                               QUrl{QStringLiteral("image://icon/folder")},
+                                               QUrl{QStringLiteral("image://icon/document-open-folder")},
                                                ViewManager::DelegateWithoutSecondaryText,
                                                ViewManager::ViewHideRating,
                                                ViewManager::IsFlatModel,
@@ -205,11 +205,6 @@ const QUrl &ViewsListData::iconUrl(int index) const
     return d->mViewsParameters[index].mMainImage;
 }
 
-ElisaUtils::IconUseColorOverlay ViewsListData::iconUseColorOverlay(int index) const
-{
-    return d->mViewsParameters[index].mIconUseColorOverlay;
-}
-
 qulonglong ViewsListData::databaseId(int index) const
 {
     return d->mViewsParameters[index].mDataFilter.databaseId();
@@ -331,7 +326,7 @@ void ViewsListData::artistsAdded(const DataTypes::ListArtistDataType &newData)
                                        oneArtist.databaseId(),
                                        ElisaUtils::Album,
                                        ElisaUtils::Artist,
-                                       QUrl{QStringLiteral("image://icon/media-optical-audio")},
+                                       QUrl{QStringLiteral("image://icon/media-default-album")},
                                        ViewManager::DelegateWithSecondaryText,
                                        ViewManager::ViewShowRating});
     }
@@ -371,11 +366,9 @@ void ViewsListData::albumsAdded(const DataTypes::ListAlbumDataType &newData)
 
     Q_EMIT dataAboutToBeAdded(d->mViewsParameters.size(), d->mViewsParameters.size() + newData.size() - 1);
     for (const auto &oneAlbum : newData) {
-        ElisaUtils::IconUseColorOverlay useColorOverlay = ElisaUtils::DoNotUseColorOverlay;
         auto coverImage = oneAlbum.albumArtURI();
         if (!coverImage.isValid()) {
             coverImage = d->mDefaultIcons[d->mEmbeddedCategory];
-            useColorOverlay = ElisaUtils::UseColorOverlay;
         }
 
         d->mViewsParameters.push_back({oneAlbum.title(),
@@ -393,8 +386,7 @@ void ViewsListData::albumsAdded(const DataTypes::ListAlbumDataType &newData)
                                        {QStringLiteral("A-Z"), QStringLiteral("Z-A")},
                                        ViewManager::SingleAlbum,
                                        ViewManager::DiscHeaders,
-                                       ViewManager::IsTrack,
-                                       useColorOverlay});
+                                       ViewManager::IsTrack});
     }
     Q_EMIT dataAdded();
 
@@ -432,10 +424,8 @@ void ViewsListData::albumModified(const DataTypes::AlbumDataType &modifiedAlbum)
             data.mMainTitle = modifiedAlbum.title();
             if (modifiedAlbum.albumArtURI().isValid()) {
                 data.mMainImage = modifiedAlbum.albumArtURI();
-                data.mIconUseColorOverlay = ElisaUtils::DoNotUseColorOverlay;
             } else {
                 data.mMainImage = d->mDefaultIcons[d->mEmbeddedCategory];
-                data.mIconUseColorOverlay = ElisaUtils::UseColorOverlay;
             }
             data.mSecondaryTitle = modifiedAlbum.artist();
 
@@ -566,7 +556,7 @@ void ViewsListData::refreshEmbeddedCategory()
                                        {i18n("Title"), i18n("Artist"), i18n("Genre"), i18n("Year")},
                                        Qt::AscendingOrder,
                                        {i18n("A-Z"), i18n("Z-A"), i18n("A-Z"), i18n("Z-A"), i18n("A-Z"), i18n("Z-A"), i18n("Oldest First"), i18n("Newest First")},
-                                       QUrl{QStringLiteral("image://icon/media-optical-audio")},
+                                       QUrl{QStringLiteral("image://icon/media-default-album")},
                                        ViewManager::DelegateWithSecondaryText,
                                        ViewManager::ViewShowRating,
                                        ViewManager::IsFlatModel});
